@@ -1,4 +1,4 @@
-//go:generate go run golang.org/x/text/cmd/gotext -srclang=en update -out=zgotext.go -lang=cs,da,de,en,es,fr,it,nl,pt,sl,ukr
+//go:generate go run golang.org/x/text/cmd/gotext@latest -srclang=en update -out=zgotext.go -lang=cs,da,de,en,es,fr,it,nl,pt,sl,ukr
 
 // Package client implements the public interface for creating eduVPN/Let's Connect! clients
 package client
@@ -6,10 +6,12 @@ package client
 import (
 	"context"
 	"errors"
+	"log/slog"
+	"net"
 	"sync"
 	"time"
 
-	"codeberg.org/eduVPN/eduvpn-common/i18nerr"
+	"codeberg.org/eduVPN/eduvpn-common/i18n/err"
 	"codeberg.org/eduVPN/eduvpn-common/internal/api"
 	"codeberg.org/eduVPN/eduvpn-common/internal/config"
 	"codeberg.org/eduVPN/eduvpn-common/internal/discovery"
@@ -23,6 +25,31 @@ import (
 	"github.com/jwijenbergh/eduoauth-go"
 )
 
+// CalculateGateway takes a CIDR encoded subnet `cidr` and returns the gateway and an error
+// TODO: move this somewhere else?
+func CalculateGateway(cidr string) (string, error) {
+	_, ipn, err := net.ParseCIDR(cidr)
+	if err != nil {
+		return "", i18nerr.WrapInternalf(err, "failed to parse CIDR for calculating gateway: %v", cidr)
+	}
+
+	ret := make(net.IP, len(ipn.IP))
+	copy(ret, ipn.IP)
+
+	for i := len(ret) - 1; i >= 0; i-- {
+		ret[i]++
+		if ret[i] > 0 {
+			break
+		}
+	}
+
+	if !ipn.Contains(ret) {
+		return "", i18nerr.Newf("IP network does not contain incremented IP: %v", ret)
+	}
+
+	return ret.String(), nil
+}
+
 // Client is the main struct for the VPN client.
 type Client struct {
 	// The name of the client
@@ -34,9 +61,6 @@ type Client struct {
 	// The fsm
 	FSM fsm.FSM
 
-	// Whether to enable debugging
-	Debug bool
-
 	// TokenSetter sets the tokens in the client
 	TokenSetter func(sid string, stype srvtypes.Type, tok srvtypes.Tokens)
 
@@ -46,12 +70,15 @@ type Client struct {
 	// tokenCacher
 	tokCacher TokenCacher
 
+	// logr is the log file rotater
+	logr *log.FileRotater
+
 	// cfg is the config
 	cfg *config.Config
 
 	mu sync.Mutex
 
-	discoMan *discovery.Manager
+	disco *discovery.Discovery
 }
 
 // GettingConfig is defined here to satisfy the server.Callbacks interface
@@ -101,7 +128,7 @@ func (c *Client) goTransition(id fsm.StateID) error {
 		return i18nerr.WrapInternal(err, "state transition error")
 	}
 	if !handled {
-		log.Logger.Debugf("transition not handled by the client to internal state: '%s'", GetStateName(id))
+		slog.Debug("transition not handled by the client to internal state", "state", GetStateName(id))
 	}
 	return nil
 }
@@ -113,7 +140,7 @@ func (c *Client) goTransition(id fsm.StateID) error {
 //   - debug: whether or not we want to enable debugging
 //
 // It returns an error if initialization failed, for example when discovery cannot be obtained and when there are no servers.
-func New(name string, version string, directory string, stateCallback func(FSMStateID, FSMStateID, interface{}) bool, debug bool) (c *Client, err error) {
+func New(name string, version string, directory string, stateCallback func(FSMStateID, FSMStateID, any) bool) (c *Client, err error) {
 	// We create the client by filling fields one by one
 	c = &Client{}
 
@@ -125,15 +152,11 @@ func New(name string, version string, directory string, stateCallback func(FSMSt
 		return nil, i18nerr.NewInternalf("The client registered with an invalid version: '%v'", version)
 	}
 
-	// Initialize the logger
-	lvl := log.LevelInfo
-	if debug {
-		lvl = log.LevelDebug
-	}
-
-	if err = log.Logger.Init(lvl, directory); err != nil {
+	logr, err := log.Init(slog.LevelDebug, directory)
+	if err != nil {
 		return nil, i18nerr.WrapInternalf(err, "The log file with directory: '%s' failed to initialize", directory)
 	}
+	c.logr = logr
 
 	// set client name
 	c.Name = name
@@ -144,25 +167,27 @@ func New(name string, version string, directory string, stateCallback func(FSMSt
 	// Initialize the FSM
 	c.FSM = newFSM(stateCallback)
 
-	// Debug only if given
-	c.Debug = debug
-
 	c.cfg = config.NewFromDirectory(directory)
 
 	// set the servers
 	c.Servers = server.NewServers(c.Name, c, c.cfg.V2)
 
-	c.discoMan = discovery.NewManager(c.cfg.Discovery())
+	c.disco = c.cfg.Discovery()
+
+	slog.Debug("Client registered", "name", name, "version", version, "directory", directory)
 
 	if !c.hasDiscovery() {
 		return c, nil
 	}
 
-	disco, release := c.discoMan.Discovery(true)
-	defer release()
-	disco.MarkServersExpired()
+	err = c.disco.Fill()
+	if err != nil {
+		slog.Warn("failed filling discovery cache", "error", err)
+	}
+
+	c.disco.MarkServersExpired()
 	if !c.cfg.HasSecureInternet() {
-		disco.MarkOrganizationsExpired()
+		c.disco.MarkOrganizationsExpired()
 	}
 
 	return c, nil
@@ -207,7 +232,7 @@ func (c *Client) AuthDone(id string, t srvtypes.Type) {
 	}
 	_, err = c.FSM.GoTransition(StateMain)
 	if err != nil {
-		log.Logger.Debugf("unhandled auth done main transition: %v", err)
+		slog.Debug("unhandled auth done transition", "error", err)
 	}
 	c.TrySave()
 }
@@ -222,10 +247,11 @@ func (c *Client) TokensUpdated(id string, t srvtypes.Type, tok eduoauth.Token) {
 	// Set the memory
 	err := c.tokCacher.Set(id, t, tok)
 	if err != nil {
-		log.Logger.Warningf("failed to set tokens into cache with error: %v", err)
+		slog.Warn("failed to set tokens in cache", "error", err)
 	}
 
 	if c.TokenSetter == nil {
+		slog.Debug("not updating client tokens")
 		return
 	}
 	// Update the client
@@ -247,21 +273,19 @@ func (c *Client) Register() error {
 
 // Deregister 'deregisters' the client, meaning saving the log file and the config and emptying out the client struct.
 func (c *Client) Deregister() {
-	c.discoMan.Cancel()
-
-	_, release := c.discoMan.Discovery(false)
 	// save the config
 	c.TrySave()
 
 	// Move the state machine back
 	_, err := c.FSM.GoTransition(StateDeregistered)
 	if err != nil {
-		log.Logger.Debugf("failed deregistered transition: %v", err)
+		slog.Debug("failed deregistered transition", "error", err)
 	}
 
 	// Close the log file
-	_ = log.Logger.Close()
-	release()
+	if c.logr != nil {
+		_ = c.logr.Close()
+	}
 
 	// Empty out the state
 	*c = Client{}
@@ -287,9 +311,7 @@ func (c *Client) ExpiryTimes() (*srvtypes.Expiry, error) {
 }
 
 func (c *Client) locationCallback(ck *cookie.Cookie, orgID string) error {
-	disco, release := c.discoMan.Discovery(false)
-	locs := disco.SecureLocationList()
-	release()
+	locs := c.disco.SecureLocationList()
 	errChan := make(chan error)
 	go func() {
 		err := c.FSM.GoTransitionRequired(StateAskLocation, &srvtypes.RequiredAskTransition{
@@ -316,14 +338,13 @@ func (c *Client) locationCallback(ck *cookie.Cookie, orgID string) error {
 // TrySave tries to save the internal state file
 // If an error occurs it logs it
 func (c *Client) TrySave() {
-	log.Logger.Debugf("saving state file")
 	if c.cfg == nil {
-		log.Logger.Warningf("no state file to save")
+		slog.Warn("no state file to save")
 		return
 	}
 	err := c.cfg.Save()
 	if err != nil {
-		log.Logger.Warningf("failed to save state file: %v", err)
+		slog.Warn("failed to save state file", "error", err)
 	}
 }
 
@@ -332,6 +353,7 @@ func (c *Client) AddServer(ck *cookie.Cookie, identifier string, _type srvtypes.
 	c.mu.Lock()
 	defer c.mu.Unlock()
 
+	slog.Debug("client wants to add a server", "id", identifier, "type", _type)
 	if !c.hasDiscovery() && _type != srvtypes.TypeCustom {
 		return i18nerr.NewInternalf("Adding a non-custom server when the client does not use discovery is not supported, identifier: %s, type: %v", identifier, _type)
 	}
@@ -368,12 +390,12 @@ func (c *Client) AddServer(ck *cookie.Cookie, identifier string, _type srvtypes.
 
 	switch _type {
 	case srvtypes.TypeInstituteAccess:
-		err = c.Servers.AddInstitute(ck.Context(), c.discoMan, identifier, ot)
+		err = c.Servers.AddInstitute(ck.Context(), c.disco, identifier, ot)
 		if err != nil {
 			return i18nerr.Wrapf(err, "Failed to add an institute access server with URL: '%s'", identifier)
 		}
 	case srvtypes.TypeSecureInternet:
-		err = c.Servers.AddSecure(ck.Context(), c.discoMan, identifier, ot)
+		err = c.Servers.AddSecure(ck.Context(), c.disco, identifier, ot)
 		if err != nil {
 			return i18nerr.Wrapf(err, "Failed to add a secure internet server with organisation ID: '%s'", identifier)
 		}
@@ -407,6 +429,8 @@ func (c *Client) GetConfig(ck *cookie.Cookie, identifier string, _type srvtypes.
 	defer c.mu.Unlock()
 	previousState := c.FSM.Current
 
+	slog.Debug("client wants to get a VPN config", "server_id", identifier, "server_type", _type, "prefer_tcp", pTCP, "startup", startup)
+
 	if !c.hasDiscovery() && _type != srvtypes.TypeCustom {
 		return nil, i18nerr.NewInternalf("Getting a non-custom server when the client does not use discovery is not supported, identifier: %s, type: %d", identifier, _type)
 	}
@@ -429,44 +453,39 @@ func (c *Client) GetConfig(ck *cookie.Cookie, identifier string, _type srvtypes.
 	}
 	err = c.GettingConfig()
 	if err != nil {
-		log.Logger.Debugf("failed getting config transition: %v", err)
+		slog.Debug("failed getting config transition", "error", err)
 	}
 
 	tok, err := c.retrieveTokens(identifier, _type)
 	if err != nil {
-		log.Logger.Debugf("no tokens found for server: '%s', with error: '%v'", identifier, err)
+		slog.Debug("no tokens found for server", "server", identifier, "error", err)
 	}
 
 	ctx := ck.Context()
 	if _type != srvtypes.TypeCustom {
-		disco, release := c.discoMan.Discovery(true)
-		// make sure the servers are fetched fresh
-		_, _, dserverr := disco.Servers(ctx)
+		_, _, dserverr := c.disco.Servers(ctx, false)
 		if dserverr != nil {
-			log.Logger.Warningf("failed to fetch server discovery when getting config: %v", dserverr)
+			slog.Warn("failed to fetch server discovery when getting config", "error", dserverr)
 		}
-		release()
 	}
 
 	var srv *server.Server
 	switch _type {
 	case srvtypes.TypeInstituteAccess:
-		srv, err = c.Servers.GetInstitute(ctx, identifier, c.discoMan, tok, startup)
+		srv, err = c.Servers.GetInstitute(ctx, identifier, c.disco, tok, startup)
 	case srvtypes.TypeSecureInternet:
-		disco, release := c.discoMan.Discovery(true)
 		// make sure the organizations are fetched if they need an update
-		_, _, dorgerr := disco.Organizations(ctx)
+		_, _, dorgerr := c.disco.Organizations(ctx, false)
 		if dorgerr != nil {
-			log.Logger.Warningf("failed to fetch organization discovery when getting config: %v", dorgerr)
+			slog.Warn("failed to fetch organization discovery when getting config", "error", dorgerr)
 		}
-		release()
-		srv, err = c.Servers.GetSecure(ctx, identifier, c.discoMan, tok, startup)
+		srv, err = c.Servers.GetSecure(ctx, identifier, c.disco, tok, startup)
 
 		var cErr *discovery.ErrCountryNotFound
 		if errors.As(err, &cErr) {
 			err = c.locationCallback(ck, identifier)
 			if err == nil {
-				srv, err = c.Servers.GetSecure(ctx, identifier, c.discoMan, tok, startup)
+				srv, err = c.Servers.GetSecure(ctx, identifier, c.disco, tok, startup)
 			}
 		}
 	case srvtypes.TypeCustom:
@@ -493,6 +512,7 @@ func (c *Client) GetConfig(ck *cookie.Cookie, identifier string, _type srvtypes.
 
 // RemoveServer removes a server
 func (c *Client) RemoveServer(identifier string, _type srvtypes.Type) (err error) {
+	slog.Debug("client wants to remove a server", "id", identifier, "type", _type)
 	identifier, err = c.convertIdentifier(identifier, _type)
 	if err != nil {
 		return err
@@ -501,10 +521,8 @@ func (c *Client) RemoveServer(identifier string, _type srvtypes.Type) (err error
 	if err != nil {
 		return i18nerr.WrapInternalf(err, "Failed to remove server: '%s'", identifier)
 	}
-	disco, release := c.discoMan.Discovery(true)
-	defer release()
 	if _type == srvtypes.TypeSecureInternet {
-		disco.MarkOrganizationsExpired()
+		c.disco.MarkOrganizationsExpired()
 	}
 	c.TrySave()
 	return nil
@@ -512,7 +530,7 @@ func (c *Client) RemoveServer(identifier string, _type srvtypes.Type) (err error
 
 // CurrentServer gets the current server that is configured
 func (c *Client) CurrentServer() (*srvtypes.Current, error) {
-	curr, err := c.Servers.PublicCurrent(c.discoMan)
+	curr, err := c.Servers.PublicCurrent(c.disco)
 	if err != nil {
 		return nil, i18nerr.WrapInternal(err, "The current server could not be retrieved")
 	}
@@ -521,6 +539,7 @@ func (c *Client) CurrentServer() (*srvtypes.Current, error) {
 
 // SetProfileID set the profile ID `pID` for the current server
 func (c *Client) SetProfileID(pID string) error {
+	slog.Debug("Client sets profile ID", "id", pID)
 	srv, err := c.Servers.CurrentServer()
 	if err != nil {
 		return i18nerr.WrapInternalf(err, "Failed to set the profile ID: '%s'", pID)
@@ -553,6 +572,7 @@ func (c *Client) retrieveTokens(sid string, t srvtypes.Type) (*eduoauth.Token, e
 
 // Cleanup cleans up the VPN connection by sending a /disconnect
 func (c *Client) Cleanup(ck *cookie.Cookie) error {
+	slog.Debug("client wants to cleanup VPN connection")
 	defer c.TrySave()
 	srv, err := c.Servers.CurrentServer()
 	if err != nil {
@@ -562,7 +582,7 @@ func (c *Client) Cleanup(ck *cookie.Cookie) error {
 	if err != nil {
 		return i18nerr.WrapInternal(err, "No OAuth tokens were found when cleaning up the connection")
 	}
-	auth, err := srv.ServerWithCallbacks(ck.Context(), c.discoMan, tok, true)
+	auth, err := srv.ServerWithCallbacks(ck.Context(), c.disco, tok, true)
 	if err != nil {
 		return i18nerr.WrapInternal(err, "The server was unable to be retrieved when cleaning up the connection")
 	}
@@ -584,6 +604,7 @@ func (c *Client) SetSecureLocation(orgID string, countryCode string) error {
 	if err != nil {
 		return i18nerr.WrapInternalf(err, "Failed to get the secure internet server with id: '%s' for setting a location", orgID)
 	}
+	slog.Debug("setting secure internet location", "org_id", orgID, "code", countryCode)
 	srv.CountryCode = countryCode
 	defer c.TrySave()
 
@@ -602,6 +623,7 @@ func (c *Client) SetSecureLocation(orgID string, countryCode string) error {
 // RenewSession is called when the user clicks on the renew session button
 // It re-authorized the server by getting a server without passing tokens
 func (c *Client) RenewSession(ck *cookie.Cookie) error {
+	slog.Debug("client wants to renew the session")
 	// getting the current serving with nil tokens means re-authorize
 	srv, err := c.Servers.CurrentServer()
 	if err != nil {
@@ -613,7 +635,7 @@ func (c *Client) RenewSession(ck *cookie.Cookie) error {
 	previousState := c.FSM.Current
 
 	// getting a server with no tokens means re-authorize
-	_, err = srv.ServerWithCallbacks(ck.Context(), c.discoMan, nil, false)
+	_, err = srv.ServerWithCallbacks(ck.Context(), c.disco, nil, false)
 	if err != nil {
 		c.FSM.GoTransition(previousState) //nolint:errcheck
 		return i18nerr.WrapInternal(err, "The server was unable to be retrieved when renewing the session")
@@ -624,6 +646,7 @@ func (c *Client) RenewSession(ck *cookie.Cookie) error {
 // StartFailover starts the failover procedure
 func (c *Client) StartFailover(ck *cookie.Cookie, gateway string, mtu int, readRxBytes func() (int64, error)) (bool, error) {
 	f := failover.New(readRxBytes)
+	slog.Debug("client wants to start the failover test", "gateway", gateway, "mtu", mtu)
 
 	// get current profile
 	d, err := f.Start(ck.Context(), gateway, mtu)
@@ -635,8 +658,6 @@ func (c *Client) StartFailover(ck *cookie.Cookie, gateway string, mtu int, readR
 
 // ServerList gets the list of servers
 func (c *Client) ServerList() (*srvtypes.List, error) {
-	disco, release := c.discoMan.Discovery(false)
-	defer release()
-	g := c.cfg.V2.PublicList(disco)
+	g := c.cfg.V2.PublicList(c.disco)
 	return g, nil
 }
