@@ -11,12 +11,88 @@ import (
 	"time"
 
 	httpw "codeberg.org/eduVPN/eduvpn-common/internal/http"
+	"codeberg.org/eduVPN/eduvpn-common/internal/test"
 	"codeberg.org/eduVPN/eduvpn-common/types/cookie"
 	"codeberg.org/eduVPN/eduvpn-common/types/protocol"
 	srvtypes "codeberg.org/eduVPN/eduvpn-common/types/server"
 	"github.com/jwijenbergh/eduoauth-go"
 )
 
+func TestCalculateGateway(t *testing.T) {
+	cases := []struct {
+		in   string
+		want string
+		err  string
+	}{
+		// normal cases
+		{
+			in:   "10.10.10.5/24",
+			want: "10.10.10.1",
+			err:  "",
+		},
+		{
+			in:   "10.10.10.130/25",
+			want: "10.10.10.129",
+			err:  "",
+		},
+		{
+			in:   "fd42::5/112",
+			want: "fd42::1",
+			err:  "",
+		},
+		{
+			in:   "5502:df9::/64",
+			want: "5502:df9::1",
+			err:  "",
+		},
+		// unrealistic scenario but we have to handle these!
+		{
+			in:   "5502:df9::0/128",
+			want: "",
+			err:  "IP network does not contain incremented IP: 5502:df9::1",
+		},
+		{
+			in:   "5502:df9::ffff/128",
+			want: "",
+			err:  "IP network does not contain incremented IP: 5502:df9::1:0",
+		},
+		{
+			in:   "10.0.0.0/32",
+			want: "",
+			err:  "IP network does not contain incremented IP: 10.0.0.1",
+		},
+		{
+			in:   "10.0.0.255/32",
+			want: "",
+			err:  "IP network does not contain incremented IP: 10.0.1.0",
+		},
+		// parsing errors
+		{
+			in:   "10.0.0.1",
+			want: "",
+			err:  "An internal error occurred. The cause of the error is: invalid CIDR address: 10.0.0.1.",
+		},
+		{
+			in:   "bla",
+			want: "",
+			err:  "An internal error occurred. The cause of the error is: invalid CIDR address: bla.",
+		},
+		{
+			in:   "5502:df9::ffff",
+			want: "",
+			err:  "An internal error occurred. The cause of the error is: invalid CIDR address: 5502:df9::ffff.",
+		},
+	}
+
+	for _, c := range cases {
+		got, err := CalculateGateway(c.in)
+		test.AssertError(t, err, c.err)
+		if got != c.want {
+			t.Fatalf("got: %v not equal to want: %v", got, c.want)
+		}
+	}
+}
+
 func getServerURI(t *testing.T) string {
 	serverURI := os.Getenv("SERVER_URI")
 	if serverURI == "" {
@@ -60,7 +136,7 @@ func stateCallback(
 	ck *cookie.Cookie,
 	_ FSMStateID,
 	newState FSMStateID,
-	data interface{},
+	data any,
 ) {
 	if newState == StateOAuthStarted {
 		url, ok := data.(string)
@@ -82,7 +158,7 @@ func TestServer(t *testing.T) {
 		"org.letsconnect-vpn.app.linux",
 		"0.1.0-test",
 		dir,
-		func(oldState FSMStateID, newState FSMStateID, data interface{}) bool {
+		func(oldState FSMStateID, newState FSMStateID, data any) bool {
 			// test if main server server list succeeds
 			if newState == StateMain {
 				_, listErr := state.ServerList()
@@ -93,7 +169,6 @@ func TestServer(t *testing.T) {
 			go stateCallback(ck, oldState, newState, data)
 			return true
 		},
-		false,
 	)
 	if err != nil {
 		t.Fatalf("Creating client error: %v", err)
@@ -137,11 +212,10 @@ func TestTokenExpired(t *testing.T) {
 		"org.letsconnect-vpn.app.linux",
 		"0.1.0-test",
 		dir,
-		func(oldState FSMStateID, newState FSMStateID, data interface{}) bool {
+		func(oldState FSMStateID, newState FSMStateID, data any) bool {
 			go stateCallback(ck, oldState, newState, data)
 			return true
 		},
-		false,
 	)
 	if err != nil {
 		t.Fatalf("Creating client error: %v", err)
@@ -196,11 +270,10 @@ func TestInvalidProfileCorrected(t *testing.T) {
 		"org.letsconnect-vpn.app.linux",
 		"0.1.0-test",
 		dir,
-		func(oldState FSMStateID, newState FSMStateID, data interface{}) bool {
+		func(oldState FSMStateID, newState FSMStateID, data any) bool {
 			go stateCallback(ck, oldState, newState, data)
 			return true
 		},
-		false,
 	)
 	if err != nil {
 		t.Fatalf("Creating client error: %v", err)
@@ -253,11 +326,10 @@ func TestConfigStartup(t *testing.T) {
 		"org.letsconnect-vpn.app.linux",
 		"0.1.0-test",
 		dir,
-		func(oldState FSMStateID, newState FSMStateID, data interface{}) bool {
+		func(oldState FSMStateID, newState FSMStateID, data any) bool {
 			go stateCallback(ck, oldState, newState, data)
 			return true
 		},
-		false,
 	)
 	if err != nil {
 		t.Fatalf("Creating client error: %v", err)
@@ -314,11 +386,10 @@ func TestPreferTCP(t *testing.T) {
 		"org.letsconnect-vpn.app.linux",
 		"0.1.0-test",
 		dir,
-		func(oldState FSMStateID, newState FSMStateID, data interface{}) bool {
+		func(oldState FSMStateID, newState FSMStateID, data any) bool {
 			go stateCallback(ck, oldState, newState, data)
 			return true
 		},
-		false,
 	)
 	if err != nil {
 		t.Fatalf("Creating client error: %v", err)
@@ -379,10 +450,9 @@ func TestInvalidClientID(t *testing.T) {
 			k,
 			"0.1.0-test",
 			dir,
-			func(_ FSMStateID, _ FSMStateID, _ interface{}) bool {
+			func(_ FSMStateID, _ FSMStateID, _ any) bool {
 				return true
 			},
-			false,
 		)
 		if v {
 			if err != nil {
