@@ -1,11 +1,10 @@
 package client
 
 import (
-	"context"
 	"sort"
 	"strings"
 
-	"codeberg.org/eduVPN/eduvpn-common/i18nerr"
+	"codeberg.org/eduVPN/eduvpn-common/i18n/err"
 	"codeberg.org/eduVPN/eduvpn-common/types/cookie"
 	discotypes "codeberg.org/eduVPN/eduvpn-common/types/discovery"
 )
@@ -19,16 +18,14 @@ func (c *Client) hasDiscovery() bool {
 // If the list cannot be retrieved an error is returned.
 // If this is the case then a previous version of the list is returned if there is any.
 // This takes into account the frequency of updates, see: https://github.com/eduvpn/documentation/blob/v3/SERVER_DISCOVERY.md#organization-list.
-func (c *Client) DiscoOrganizations(ck *cookie.Cookie, search string) (*discotypes.Organizations, error) {
+// cache is set to true if there should be no network call done
+func (c *Client) DiscoOrganizations(ck *cookie.Cookie, cache bool, search string) (*discotypes.Organizations, error) {
 	// Not supported with Let's Connect! & govVPN
 	if !c.hasDiscovery() {
 		return nil, i18nerr.NewInternal("Organization discovery with this client ID is not supported")
 	}
 
-	disco, release := c.discoMan.Discovery(true)
-	defer release()
-
-	orgs, fresh, err := disco.Organizations(ck.Context())
+	orgs, fresh, err := c.disco.Organizations(ck.Context(), cache)
 	if fresh {
 		defer c.TrySave()
 	}
@@ -68,15 +65,14 @@ func (c *Client) DiscoOrganizations(ck *cookie.Cookie, search string) (*discotyp
 // If the list cannot be retrieved an error is returned.
 // If this is the case then a previous version of the list is returned if there is any.
 // This takes into account the frequency of updates, see: https://github.com/eduvpn/documentation/blob/v3/SERVER_DISCOVERY.md#server-list.
-func (c *Client) DiscoServers(ck *cookie.Cookie, search string) (*discotypes.Servers, error) {
+// the cache argument is true if no network call should be done
+func (c *Client) DiscoServers(ck *cookie.Cookie, cache bool, search string) (*discotypes.Servers, error) {
 	// Not supported with Let's Connect! & govVPN
 	if !c.hasDiscovery() {
 		return nil, i18nerr.NewInternal("Server discovery with this client ID is not supported")
 	}
 
-	disco, release := c.discoMan.Discovery(true)
-	defer release()
-	servs, fresh, err := disco.Servers(ck.Context())
+	servs, fresh, err := c.disco.Servers(ck.Context(), cache)
 	if fresh {
 		defer c.TrySave()
 	}
@@ -111,27 +107,3 @@ func (c *Client) DiscoServers(ck *cookie.Cookie, search string) (*discotypes.Ser
 		List: retServs,
 	}, err
 }
-
-func (c *Client) DiscoveryStartup(cb func()) error {
-	// Not supported with Let's Connect! & govVPN
-	if !c.hasDiscovery() {
-		return i18nerr.NewInternal("Server/organization discovery startup with this client ID is not supported")
-	}
-
-	fcb := func() {
-		if cb == nil {
-			return
-		}
-
-		c.mu.Lock()
-		defer c.mu.Unlock()
-
-		if c.FSM.Current != StateMain {
-			return
-		}
-
-		cb()
-	}
-	c.discoMan.Startup(context.Background(), fcb)
-	return nil
-}
