@@ -2,10 +2,10 @@ package client
 
 import (
 	"fmt"
+	"log/slog"
 
-	"codeberg.org/eduVPN/eduvpn-common/i18nerr"
+	"codeberg.org/eduVPN/eduvpn-common/i18n/err"
 	"codeberg.org/eduVPN/eduvpn-common/internal/fsm"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
 )
 
 type (
@@ -90,7 +90,7 @@ func GetStateName(s FSMStateID) string {
 }
 
 func newFSM(
-	callback func(FSMStateID, FSMStateID, interface{}) bool,
+	callback func(FSMStateID, FSMStateID, any) bool,
 ) fsm.FSM {
 	states := FSMStates{
 		StateDeregistered: FSMState{
@@ -170,7 +170,14 @@ func newFSM(
 		},
 	}
 
-	return fsm.NewFSM(StateMain, states, callback, GetStateName)
+	cbLogged := func(oldState FSMStateID, newState FSMStateID, data any) bool {
+		if oldState != newState {
+			slog.Debug("FSM transition", "old", GetStateName(oldState), "new", GetStateName(newState))
+		}
+		return callback(oldState, newState, data)
+	}
+
+	return fsm.NewFSM(StateMain, states, cbLogged, GetStateName)
 }
 
 // SetState sets the state for the client FSM to `state`
@@ -182,7 +189,7 @@ func (c *Client) SetState(state FSMStateID) error {
 	if err != nil {
 		// self-transitions are only debug errors
 		if c.FSM.InState(state) {
-			log.Logger.Debugf("attempt an invalid self-transition: %s", c.FSM.GetStateName(state))
+			slog.Debug("attempt at an invalid self-transition", "transition", c.FSM.GetStateName(state))
 			return nil
 		}
 		return i18nerr.WrapInternalf(err, "Failed internal state transition requested by the client from: '%s' to '%s'", GetStateName(curr), GetStateName(state))
