@@ -1,5 +1,3 @@
-// package main implements the main exported API to be used by other languages
-//
 // Some notes:
 //
 // - Errors are returned as JSON c strings. The JSON type is defined in `types/error/error.go Error`. Free them using `FreeString`. Same is the case for other string types, you should also free them. The errors are always localized
@@ -23,19 +21,36 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"log/slog"
 	"runtime/cgo"
 	"unsafe"
 
 	"codeberg.org/eduVPN/eduvpn-common/client"
-	"codeberg.org/eduVPN/eduvpn-common/i18nerr"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
+	"codeberg.org/eduVPN/eduvpn-common/i18n/err"
 	"codeberg.org/eduVPN/eduvpn-common/proxy"
 	"codeberg.org/eduVPN/eduvpn-common/types/cookie"
 	errtypes "codeberg.org/eduVPN/eduvpn-common/types/error"
 	srvtypes "codeberg.org/eduVPN/eduvpn-common/types/server"
-	"codeberg.org/eduVPN/eduvpn-common/util"
 )
 
+// goString copies a null-terminated *C.char to a Go string.
+// the default C.GoString doesn't play well with ARM MTE
+// see: https://github.com/golang/go/issues/27610#issuecomment-2381919857
+// see also: https://codeberg.org/eduVPN/android/issues/31
+// this was changed from: https://github.com/ebitengine/purego/blob/77ed15346fbb448af291a0509e024e06252a428f/internal/strings/strings.go#L25C1-L40C2
+// License: https://github.com/ebitengine/purego/blob/77ed15346fbb448af291a0509e024e06252a428f/LICENSE
+func goString(c *C.char) string {
+	ptr := unsafe.Pointer(c)
+	if ptr == nil {
+		return ""
+	}
+	length := 0
+	for *(*byte)(unsafe.Add(ptr, uintptr(length))) != '\x00' {
+		length++
+	}
+	return C.GoStringN(c, C.int(length))
+}
+
 // VPNState is the current state of the library
 var VPNState *client.Client
 
@@ -43,9 +58,10 @@ func getCError(err error) *C.char {
 	if err == nil {
 		return nil
 	}
+	enMsg := err.Error()
 	retErr := errtypes.Error{
 		Message: errtypes.Translated{
-			"en": err.Error(),
+			"en": enMsg,
 		},
 		Misc: false,
 	}
@@ -58,6 +74,7 @@ func getCError(err error) *C.char {
 	if err != nil {
 		return C.CString(fmt.Sprintf("failed to get error return: %v", err))
 	}
+	slog.Debug("error returned to client", "error", enMsg)
 	return C.CString(retData)
 }
 
@@ -85,7 +102,7 @@ func stateCallback(
 	newStateC := C.int(newState)
 	d, err := getReturnData(data)
 	if err != nil {
-		log.Logger.Errorf("failed to get return data: %v", err)
+		slog.Error("cgo failed to get return data", "error", err)
 		return false
 	}
 	dataC := C.CString(d)
@@ -113,12 +130,6 @@ func getVPNState() (*client.Client, error) {
 //
 //   - This callback returns non-zero if the state transition is handled. This is used to check if the client handles the needed transitions
 //
-// `debug`, if non-zero, enables debugging mode for the library, this means:
-//
-//   - Log everything in debug mode, so you can get more detail of what is going on
-//
-//   - Write the state graph to a file in the configDirectory. This can be used to create a FSM png file with mermaid https://mermaid.js.org/
-//
 // After registering, the FSM is initialized and the state transition `MAIN` should have been completed
 // If some error occurs during registering, it is returned as a `types/error/error.go Error`
 //
@@ -140,20 +151,18 @@ func Register(
 	version *C.char,
 	configDirectory *C.char,
 	cb C.StateCB,
-	debug C.int,
 ) *C.char {
 	_, stateErr := getVPNState()
 	if stateErr == nil {
 		return getCError(i18nerr.NewInternal("failed to register, a VPN state is already present"))
 	}
 	c, err := client.New(
-		C.GoString(name),
-		C.GoString(version),
-		C.GoString(configDirectory),
+		goString(name),
+		goString(version),
+		goString(configDirectory),
 		func(oldState client.FSMStateID, newState client.FSMStateID, data interface{}) bool {
 			return stateCallback(cb, oldState, newState, data)
 		},
-		debug != 0,
 	)
 	// Only update the state if we get no error
 	if err == nil {
@@ -300,7 +309,7 @@ func AddServer(c C.uintptr_t, _type C.int, id *C.char, ot *C.longlong) *C.char {
 		got := int64(*ot)
 		auth = &got
 	}
-	err = state.AddServer(v, C.GoString(id), srvtypes.Type(_type), auth)
+	err = state.AddServer(v, goString(id), srvtypes.Type(_type), auth)
 	return getCError(err)
 }
 
@@ -334,7 +343,7 @@ func RemoveServer(_type C.int, id *C.char) *C.char {
 	if stateErr != nil {
 		return getCError(stateErr)
 	}
-	err := state.RemoveServer(C.GoString(id), srvtypes.Type(_type))
+	err := state.RemoveServer(goString(id), srvtypes.Type(_type))
 	return getCError(err)
 }
 
@@ -452,7 +461,7 @@ func ServerList() (*C.char, *C.char) {
 	return C.CString(ret), nil
 }
 
-// GetConfig gets a configuration for the server. It returns additional information in case WireGuard over Proxyguard is used (see the last example)
+// GetConfig gets a configuration for the server.
 //
 // `c` is the cookie that is used for cancellation. Create a cookie first with CookieNew, this same cookie is also used for replying to state transitions
 //
@@ -538,16 +547,6 @@ func ServerList() (*C.char, *C.char) {
 //	 "should_failover": true, <- whether or not the failover procedure should happen
 //	}
 //
-// Example Output (3=WireGuard + Proxyguard):
-//
-//	{
-//	"config":"[Interface]\nMTU = ...\nAddress = ...\nDNS = ...\nPrivateKey = ...\n[Peer]\nPublicKey = ...\nAllowedIPs = ...\nEndpoint = 127.0.0.1:x\n",
-//	"protocol":3,
-//	"default_gateway":true,
-//	"should_failover":true,
-//	"proxy":{"source_port":38683,"listen":"127.0.0.1:59812","peer":"https://..."}
-//	}
-//
 //export GetConfig
 func GetConfig(c C.uintptr_t, _type C.int, id *C.char, pTCP C.int, startup C.int) (*C.char, *C.char) {
 	state, stateErr := getVPNState()
@@ -560,7 +559,7 @@ func GetConfig(c C.uintptr_t, _type C.int, id *C.char, pTCP C.int, startup C.int
 	}
 	preferTCPBool := pTCP != 0
 	startupBool := startup != 0
-	cfg, err := state.GetConfig(ck, C.GoString(id), srvtypes.Type(_type), preferTCPBool, startupBool)
+	cfg, err := state.GetConfig(ck, goString(id), srvtypes.Type(_type), preferTCPBool, startupBool)
 	if cfg != nil && err == nil {
 		d, err := getReturnData(cfg)
 		if err == nil {
@@ -594,7 +593,7 @@ func SetProfileID(data *C.char) *C.char {
 	if stateErr != nil {
 		return getCError(stateErr)
 	}
-	profileErr := state.SetProfileID(C.GoString(data))
+	profileErr := state.SetProfileID(goString(data))
 	return getCError(profileErr)
 }
 
@@ -623,13 +622,14 @@ func SetSecureLocation(orgID *C.char, cc *C.char) *C.char {
 	if stateErr != nil {
 		return getCError(stateErr)
 	}
-	locationErr := state.SetSecureLocation(C.GoString(orgID), C.GoString(cc))
+	locationErr := state.SetSecureLocation(goString(orgID), goString(cc))
 	return getCError(locationErr)
 }
 
 // DiscoServers gets the servers from discovery, returned as `types/discovery/discovery.go Servers` marshalled as JSON
 //
 //   - `c` is the Cookie that needs to be passed. Create a new Cookie using `CookieNew`
+//   - `cache` indicates whether or not the cache should only be used, meaning no network call
 //   - `search` is the search string for filtering the list.
 //
 // If any of the words in the search query is not contained in any of the display names or keywords, the candidate is filtered.
@@ -673,7 +673,7 @@ func SetSecureLocation(orgID *C.char, cc *C.char) *C.char {
 //		} , null
 //
 //export DiscoServers
-func DiscoServers(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
+func DiscoServers(c C.uintptr_t, cache C.int, search *C.char) (*C.char, *C.char) {
 	state, stateErr := getVPNState()
 	if stateErr != nil {
 		return nil, getCError(stateErr)
@@ -682,7 +682,7 @@ func DiscoServers(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
 	if err != nil {
 		return nil, getCError(err)
 	}
-	servers, err := state.DiscoServers(ck, C.GoString(search))
+	servers, err := state.DiscoServers(ck, cache != 0, goString(search))
 	if servers == nil && err != nil {
 		return nil, getCError(err)
 	}
@@ -696,6 +696,7 @@ func DiscoServers(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
 // DiscoOrganizations gets the organizations from discovery, returned as `types/discovery/discovery.go Organizations` marshalled as JSON.
 //
 //   - `c` is the Cookie that needs to be passed. Create a new Cookie using `CookieNew`
+//   - `cache` indicates whether or not the cache should only be used, meaning no network call
 //   - `search` is the search string for filtering the list.
 //
 // If any of the words in the `search` query is not contained in any of the display names or keywords, the candidate is filtered.
@@ -748,7 +749,7 @@ func DiscoServers(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
 //		}, null
 //
 //export DiscoOrganizations
-func DiscoOrganizations(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
+func DiscoOrganizations(c C.uintptr_t, cache C.int, search *C.char) (*C.char, *C.char) {
 	state, stateErr := getVPNState()
 	if stateErr != nil {
 		return nil, getCError(stateErr)
@@ -757,7 +758,7 @@ func DiscoOrganizations(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
 	if err != nil {
 		return nil, getCError(err)
 	}
-	orgs, err := state.DiscoOrganizations(ck, C.GoString(search))
+	orgs, err := state.DiscoOrganizations(ck, cache != 0, goString(search))
 	if orgs == nil && err != nil {
 		return nil, getCError(err)
 	}
@@ -769,7 +770,6 @@ func DiscoOrganizations(c C.uintptr_t, search *C.char) (*C.char, *C.char) {
 }
 
 // Cleanup sends a `/disconnect` to cleanup the connection.
-// Additionally, if ProxyGuard is active it cancels the running process
 //
 // This MUST be called when disconnecting, see [the eduVPN docs](https://docs.eduvpn.org/server/v3/api.html#application-flow).
 // `c` is the Cookie that needs to be passed. Create a new Cookie using `CookieNew`.
@@ -860,7 +860,7 @@ func StartFailover(c C.uintptr_t, gateway *C.char, mtu C.int, readRxBytes C.Read
 	if err != nil {
 		return C.int(0), getCError(err)
 	}
-	dropped, droppedErr := state.StartFailover(ck, C.GoString(gateway), int(mtu), func() (int64, error) {
+	dropped, droppedErr := state.StartFailover(ck, goString(gateway), int(mtu), func() (int64, error) {
 		rxBytes := int64(C.get_read_rx_bytes(readRxBytes))
 		if rxBytes < 0 {
 			return 0, i18nerr.NewInternal("client gave an invalid rx bytes value")
@@ -1047,28 +1047,6 @@ func getCookie(c C.uintptr_t) (*cookie.Cookie, error) {
 	return v, nil
 }
 
-// DiscoveryStartup does a discovery request in the background.
-//
-//   - The `refresh` argument is a callback that is called when the refreshing is done.
-//
-// When this callback is thus called, the app SHOULD refresh the server list of the already configured servers.
-// This DiscoveryStartup function MUST be called after calling `Register`.
-//
-//export DiscoveryStartup
-func DiscoveryStartup(refresh C.RefreshList) *C.char {
-	state, stateErr := getVPNState()
-	if stateErr != nil {
-		return getCError(stateErr)
-	}
-	startupErr := state.DiscoveryStartup(func() {
-		if refresh == nil {
-			return
-		}
-		C.call_refresh_list(refresh)
-	})
-	return getCError(startupErr)
-}
-
 // SetTokenHandler sets the token getters and token setters for OAuth.
 //
 // Because the data that is saved does not contain OAuth tokens for server, the common lib asks and sets the tokens using these callback functions.
@@ -1100,7 +1078,7 @@ func SetTokenHandler(getter C.TokenGetter, setter C.TokenSetter) *C.char {
 	state.TokenSetter = func(sid string, stype srvtypes.Type, t srvtypes.Tokens) {
 		tJSON, err := getReturnData(t)
 		if err != nil {
-			log.Logger.Warningf("failed to get tokens for setting tokens in exports: %v", err)
+			slog.Warn("cgo failed to get tokens in setting tokens callback", "error", err)
 			return
 		}
 		c1 := C.CString(sid)
@@ -1122,20 +1100,20 @@ func SetTokenHandler(getter C.TokenGetter, setter C.TokenSetter) *C.char {
 		// get null pointer index as unmarshalling wants it without
 		null := bytes.IndexByte(d, 0)
 		if null < 0 {
-			log.Logger.Warningf("output buffer is not NULL terminated")
+			slog.Warn("cgo output buffer in token getter is not null terminated")
 			return nil
 		}
 
 		// no data found
 		if null == 0 {
-			log.Logger.Debugf("empty string returned when getting tokens")
+			slog.Warn("cgo empty string returned in token getter")
 			return nil
 		}
 
 		var gotT srvtypes.Tokens
 		err := json.Unmarshal(d[:null], &gotT)
 		if err != nil {
-			log.Logger.Warningf("failed to get JSON data for getting tokens in exports: %v", err)
+			slog.Warn("cgo failed to get JSON data in token getter", "error", err)
 			return nil
 		}
 		return &gotT
@@ -1155,7 +1133,7 @@ func SetTokenHandler(getter C.TokenGetter, setter C.TokenSetter) *C.char {
 //
 //export CalculateGateway
 func CalculateGateway(subnet *C.char) (*C.char, *C.char) {
-	gw, err := util.CalculateGateway(C.GoString(subnet))
+	gw, err := client.CalculateGateway(goString(subnet))
 	if err != nil {
 		return nil, getCError(err)
 	}
@@ -1196,7 +1174,7 @@ func CookieReply(c C.uintptr_t, data *C.char) *C.char {
 	if err != nil {
 		return getCError(err)
 	}
-	err = v.Send(C.GoString(data))
+	err = v.Send(goString(data))
 	return getCError(err)
 }
 
