@@ -8,7 +8,6 @@ typedef long long int (*ReadRxBytes)();
 
 typedef int (*StateCB)(int oldstate, int newstate, void* data);
 
-typedef void (*RefreshList)();
 typedef void (*TokenGetter)(const char* server_id, int server_type, char* out, size_t len);
 typedef void (*TokenSetter)(const char* server_id, int server_type, const char* tokens);
 typedef void (*ProxySetup)(int fd);
@@ -21,10 +20,6 @@ static int call_callback(StateCB callback, int oldstate, int newstate, void* dat
 {
     return callback(oldstate, newstate, data);
 }
-static void call_refresh_list(RefreshList refresh)
-{
-    refresh();
-}
 static void call_token_getter(TokenGetter getter, const char* server_id, int server_type, char* out, size_t len)
 {
     getter(server_id, server_type, out, len);
