@@ -10,6 +10,7 @@ import (
 	"net/url"
 	"reflect"
 	"regexp"
+	"slices"
 	"strings"
 	"testing"
 	"time"
@@ -17,7 +18,6 @@ import (
 	"codeberg.org/eduVPN/eduvpn-common/internal/api/profiles"
 	httpw "codeberg.org/eduVPN/eduvpn-common/internal/http"
 	"codeberg.org/eduVPN/eduvpn-common/internal/test"
-	"codeberg.org/eduVPN/eduvpn-common/internal/wireguard"
 	"codeberg.org/eduVPN/eduvpn-common/types/protocol"
 	"codeberg.org/eduVPN/eduvpn-common/types/server"
 	"github.com/jwijenbergh/eduoauth-go"
@@ -38,20 +38,18 @@ func tokenHandler(t *testing.T, gt []string) func(http.ResponseWriter, *http.Req
 		}
 		grant := parsed.Get("grant_type")
 
-		for _, v := range gt {
-			if v == grant {
-				_, err = w.Write([]byte(`
+		if slices.Contains(gt, grant) {
+			_, err = w.Write([]byte(`
 {
 	"access_token": "validaccess",
 	"refresh_token": "validrefresh",
 	"expires_in": 3600
 }
 		`))
-				if err != nil {
-					t.Fatalf("failed writing in token handler: %v", err)
-				}
-				return
+			if err != nil {
+				t.Fatalf("failed writing in token handler: %v", err)
 			}
+			return
 		}
 		t.Fatalf("grant type: %v, not allowed", grant)
 	}
@@ -260,7 +258,7 @@ func TestAPIInfo(t *testing.T) {
 	cases := []struct {
 		hp   test.HandlerPath
 		info *profiles.Info
-		err  interface{}
+		err  any
 	}{
 		{
 			hp: test.HandlerPath{
@@ -422,7 +420,6 @@ func TestAPIConnect(t *testing.T) {
 				Configuration: "openvpnconfig\nscript-security 0",
 				Protocol:      protocol.OpenVPN,
 				Expires:       time.Date(2000, time.January, 0, 0, 0, 0, 0, time.UTC),
-				Proxy:         nil,
 			},
 			protos: []protocol.Protocol{protocol.OpenVPN, protocol.WireGuard},
 			err:    nil,
@@ -438,7 +435,6 @@ func TestAPIConnect(t *testing.T) {
 PrivateKey = .*`,
 				Protocol: protocol.WireGuard,
 				Expires:  time.Date(2000, time.January, 0, 0, 0, 0, 0, time.UTC),
-				Proxy:    nil,
 			},
 			protos: []protocol.Protocol{protocol.OpenVPN, protocol.WireGuard},
 			err:    nil,
@@ -454,8 +450,6 @@ PrivateKey = .*`,
 PrivateKey = .*`,
 				Protocol: protocol.WireGuardProxy,
 				Expires:  time.Date(2000, time.January, 0, 0, 0, 0, 0, time.UTC),
-				// proxy will be manually checked
-				Proxy: &wireguard.Proxy{},
 			},
 			ptcp:   true,
 			protos: []protocol.Protocol{protocol.OpenVPN, protocol.WireGuard},
@@ -491,19 +485,6 @@ PrivateKey = .*`,
 			// we have already checked the config using a regex
 			c.cd.Configuration = gcd.Configuration
 
-			// check proxy manually
-			if c.cd.Proxy != nil && gcd.Proxy != nil {
-				if gcd.Proxy.Peer != "https://proxyendpoint" {
-					t.Fatalf("config data proxy peer is no proxyendpoint with HTTPS scheme: %s", gcd.Proxy.Peer)
-				}
-				if gcd.Proxy.SourcePort <= 0 {
-					t.Fatalf("got proxy source port is smaller or equal to 0: %v", gcd.Proxy.SourcePort)
-				}
-				if gcd.Proxy.ListenPort <= 0 {
-					t.Fatalf("proxy listen port is smaller or equal to 0: %v", gcd.Proxy.ListenPort)
-				}
-				c.cd.Proxy = gcd.Proxy
-			}
 		}
 		if !reflect.DeepEqual(gcd, c.cd) {
 			t.Fatalf("got connect data: %v, not equal to want: %v", gcd, c.cd)
