@@ -4,15 +4,13 @@ package config
 
 import (
 	"encoding/json"
+	"log/slog"
 	"os"
 	"path"
 
-	"codeberg.org/eduVPN/eduvpn-common/internal/config/atomicfile"
-	"codeberg.org/eduVPN/eduvpn-common/internal/config/v1"
+	"codeberg.org/eduVPN/eduvpn-common/internal/atomicfile"
 	"codeberg.org/eduVPN/eduvpn-common/internal/config/v2"
 	"codeberg.org/eduVPN/eduvpn-common/internal/discovery"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
-	"codeberg.org/eduVPN/eduvpn-common/internal/util"
 )
 
 const stateFile = "state.json"
@@ -40,7 +38,7 @@ func (c *Config) HasSecureInternet() bool {
 
 // Save saves the state file to disk
 func (c *Config) Save() error {
-	if err := util.EnsureDirectory(c.directory); err != nil {
+	if err := os.MkdirAll(c.directory, 0o700); err != nil {
 		return err
 	}
 
@@ -65,20 +63,12 @@ func (c *Config) Load() error {
 	if err = json.Unmarshal(bts, &buf); err != nil {
 		return err
 	}
-	if buf.V2 != nil {
-		c.V2 = buf.V2
-		return nil
-	}
-	if buf.V1 != nil {
-		c.V2 = v2.FromV1(buf.V1)
-	}
+	c.V2 = buf.V2
 	return nil
 }
 
 // Versioned is the final top-level state file that is written to disk
 type Versioned struct {
-	// V1 is the version 1 state file that is no longer used but converted from
-	V1 *v1.V1 `json:"v1,omitempty"`
 	// V2 is the version 2 state file
 	V2 *v2.V2 `json:"v2,omitempty"`
 }
@@ -91,7 +81,7 @@ func NewFromDirectory(dir string) *Config {
 	}
 	err := cfg.Load()
 	if err != nil {
-		log.Logger.Debugf("failed to load configuration: %v", err)
+		slog.Debug("failed to load configuration", "error", err)
 	}
 	if cfg.V2 == nil {
 		cfg.V2 = &v2.V2{}
