@@ -4,11 +4,11 @@ package v2
 import (
 	"errors"
 	"fmt"
+	"log/slog"
 	"net/url"
 	"time"
 
 	"codeberg.org/eduVPN/eduvpn-common/internal/discovery"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
 	"codeberg.org/eduVPN/eduvpn-common/types/server"
 )
 
@@ -215,7 +215,7 @@ func convertCustom(u string) server.Server {
 	dn := u
 	pu, err := url.Parse(u)
 	if err != nil {
-		log.Logger.Errorf("failed to parse server hostname: %v", err)
+		slog.Error("failed to parse server hostname", "error", err)
 	} else {
 		dn = pu.Hostname()
 	}
@@ -272,7 +272,7 @@ func (cfg *V2) PublicList(disco *discovery.Discovery) *server.List {
 			g.Profiles = v.Profiles
 			ret.Custom = append(ret.Custom, g)
 		default:
-			log.Logger.Errorf("no such server type in list: '%v'", k.T)
+			slog.Error("no such server type in list", "type", k.T)
 			continue
 		}
 	}
