@@ -6,12 +6,13 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"log/slog"
 	"net/http"
+	"sync"
 	"time"
 
 	httpw "codeberg.org/eduVPN/eduvpn-common/internal/http"
 	"codeberg.org/eduVPN/eduvpn-common/internal/levenshtein"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
 	"codeberg.org/eduVPN/eduvpn-common/internal/verify"
 	discotypes "codeberg.org/eduVPN/eduvpn-common/types/discovery"
 )
@@ -44,6 +45,7 @@ type Organization struct {
 	KeywordList discotypes.MapOrString `json:"keyword_list,omitempty"`
 }
 
+// Score returns the levenshstein score for a search query `search` on the organizations
 func (o *Organization) Score(search string) int {
 	return levenshtein.DiscoveryScore(search, o.DisplayName, o.KeywordList)
 }
@@ -75,13 +77,15 @@ type Server struct {
 	SupportContact []string `json:"support_contact,omitempty"`
 }
 
-// Matches returns if the search query `str` matches with this server
+// Score returns the score of the search query `str` on this server
 func (s *Server) Score(search string) int {
 	return levenshtein.DiscoveryScore(search, s.DisplayName, s.KeywordList)
 }
 
 // Discovery is the main structure used for this package.
 type Discovery struct {
+	// mu is the read write mutex that protects the struct from concurrent access
+	mu sync.RWMutex
 	// The httpClient for sending HTTP requests
 	httpClient *httpw.Client
 
@@ -97,7 +101,7 @@ var DiscoURL = "https://disco.eduvpn.org/v2/"
 
 // file is a helper function that gets a disco JSON and fills the structure with it
 // If it was unsuccessful it returns an error.
-func (discovery *Discovery) file(ctx context.Context, jsonFile string, previousVersion uint64, last time.Time, structure interface{}) (time.Time, error) {
+func (discovery *Discovery) file(ctx context.Context, jsonFile string, previousVersion uint64, last time.Time, structure any) (time.Time, error) {
 	var newUpdate time.Time
 	// No HTTP client present, create one
 	if discovery.httpClient == nil {
@@ -128,13 +132,13 @@ func (discovery *Discovery) file(ctx context.Context, jsonFile string, previousV
 	if lms != "" {
 		lm, err := http.ParseTime(lms)
 		if err != nil {
-			log.Logger.Warningf("failed to parse 'Last-Modified' header: %v", err)
+			slog.Warn("failed to parse 'Last-Modified' header", "error", err)
 		} else {
 			newUpdate = lm
-			log.Logger.Debugf("got 'Last-Modified' header: %v", lm)
+			slog.Debug("got 'Last-Modified' header", "value", lm)
 		}
 	} else {
-		log.Logger.Warningf("no 'Last-Modified' header found")
+		slog.Warn("no 'Last-Modified' header found")
 	}
 
 	// Get signature
@@ -173,23 +177,27 @@ func (discovery *Discovery) file(ctx context.Context, jsonFile string, previousV
 
 // MarkOrganizationsExpired marks the organizations as expired
 func (discovery *Discovery) MarkOrganizationsExpired() {
+	discovery.mu.Lock()
+	defer discovery.mu.Unlock()
 	// Re-initialize the timestamp to zero
 	discovery.OrganizationList.Timestamp = time.Time{}
 }
 
 // MarkServersExpired marks the servers as expired
 func (discovery *Discovery) MarkServersExpired() {
+	discovery.mu.Lock()
+	defer discovery.mu.Unlock()
 	// Re-initialize the timestamp to zero
 	discovery.ServerList.Timestamp = time.Time{}
 }
 
-// DetermineOrganizationsUpdate returns a boolean indicating whether or not the discovery organizations should be updated
+// determineOrganizationsUpdate returns a boolean indicating whether or not the discovery organizations should be updated
 // https://github.com/eduvpn/documentation/blob/v3/SERVER_DISCOVERY.md
 // - [IMPLEMENTED] on "first launch" when offering the search for "Institute Access" and "Organizations";
 // - [IMPLEMENTED in client/client.go and here] when the user tries to add new server AND the user did NOT yet choose an organization before; Implemented in Register()
 // - [IMPLEMENTED in client/client.go] when the authorization for the server associated with an already chosen organization is triggered, e.g. after expiry or revocation.
 // - [IMPLEMENTED here] NOTE: when the org_id that the user chose previously is no longer available in organization_list.json the application should ask the user to choose their organization (again). This can occur for example when the organization replaced their identity provider, uses a different domain after rebranding or simply ceased to exist.
-func (discovery *Discovery) DetermineOrganizationsUpdate() bool {
+func (discovery *Discovery) determineOrganizationsUpdate() bool {
 	if discovery.OrganizationList.Timestamp.IsZero() {
 		return true
 	}
@@ -201,6 +209,8 @@ func (discovery *Discovery) DetermineOrganizationsUpdate() bool {
 
 // SecureLocationList returns a slice of all the available locations.
 func (discovery *Discovery) SecureLocationList() []string {
+	discovery.mu.RLock()
+	defer discovery.mu.RUnlock()
 	var loc []string
 	for _, srv := range discovery.ServerList.List {
 		if srv.Type == "secure_internet" {
@@ -216,6 +226,8 @@ func (discovery *Discovery) ServerByURL(
 	baseURL string,
 	srvType string,
 ) (*Server, error) {
+	discovery.mu.RLock()
+	defer discovery.mu.RUnlock()
 	for _, currentServer := range discovery.ServerList.List {
 		if currentServer.BaseURL == baseURL && currentServer.Type == srvType {
 			return &currentServer, nil
@@ -236,6 +248,8 @@ func (cnf *ErrCountryNotFound) Error() string {
 // ServerByCountryCode returns the discovery server by the country code
 // An error is returned if and only if nil is returned for the server.
 func (discovery *Discovery) ServerByCountryCode(countryCode string) (*Server, error) {
+	discovery.mu.RLock()
+	defer discovery.mu.RUnlock()
 	for _, srv := range discovery.ServerList.List {
 		if srv.CountryCode == countryCode && srv.Type == "secure_internet" {
 			return &srv, nil
@@ -260,8 +274,10 @@ func (discovery *Discovery) orgByID(orgID string) (*Organization, error) {
 // - The secure internet server itself
 // An error is returned if and only if nil is returned for the organization.
 func (discovery *Discovery) SecureHomeArgs(orgID string) (*Organization, *Server, error) {
+	discovery.mu.RLock()
 	org, err := discovery.orgByID(orgID)
 	if err != nil {
+		discovery.mu.RUnlock()
 		discovery.MarkOrganizationsExpired()
 		return nil, nil, err
 	}
@@ -269,17 +285,19 @@ func (discovery *Discovery) SecureHomeArgs(orgID string) (*Organization, *Server
 	// Get a server with the base url
 	srv, err := discovery.ServerByURL(org.SecureInternetHome, "secure_internet")
 	if err != nil {
+		discovery.mu.RUnlock()
 		discovery.MarkOrganizationsExpired()
 		return nil, nil, err
 	}
+	discovery.mu.RUnlock()
 	return org, srv, nil
 }
 
-// DetermineServersUpdate returns whether or not the discovery servers should be updated by contacting the discovery server
+// determineServersUpdate returns whether or not the discovery servers should be updated by contacting the discovery server
 // https://github.com/eduvpn/documentation/blob/v3/SERVER_DISCOVERY.md
 // - [Implemented] The application MUST always fetch the server_list.json at application start.
 // - The application MAY refresh the server_list.json periodically, e.g. once every hour.
-func (discovery *Discovery) DetermineServersUpdate() bool {
+func (discovery *Discovery) determineServersUpdate() bool {
 	// No servers, we should update
 	if discovery.ServerList.Timestamp.IsZero() {
 		return true
@@ -292,45 +310,26 @@ func (discovery *Discovery) DetermineServersUpdate() bool {
 	return !time.Now().Before(upd)
 }
 
-func (discovery *Discovery) previousOrganizations() (*Organizations, error) {
-	// If the version field is not zero then we have a cached struct
-	// We also immediately return this copy if we have no embedded JSON
-	if discovery.OrganizationList.Version != 0 || !HasCache {
-		return &discovery.OrganizationList, nil
+func (discovery *Discovery) cachedOrgs() *Organizations {
+	if discovery.OrganizationList.Version == 0 {
+		return nil
 	}
-
-	// We do not have a cached struct, this we need to get it using the embedded JSON
-	var eo Organizations
-	if err := json.Unmarshal(eOrganizations, &eo); err != nil {
-		return nil, fmt.Errorf("failed parsing discovery organizations from the embedded cache with error: %w", err)
-	}
-	discovery.OrganizationList = eo
-	return &eo, nil
-}
-
-func (discovery *Discovery) previousServers() (*Servers, error) {
-	// If the version field is not zero then we have a cached struct
-	// We also immediately return this copy if we have no embedded JSON
-	if discovery.ServerList.Version != 0 || !HasCache {
-		return &discovery.ServerList, nil
-	}
-
-	// We do not have a cached struct, this we need to get it using the embedded JSON
-	var es Servers
-	if err := json.Unmarshal(eServers, &es); err != nil {
-		return nil, fmt.Errorf("failed parsing discovery servers from the embedded cache with error: %w", err)
-	}
-	discovery.ServerList = es
-	return &es, nil
+	return &discovery.OrganizationList
 }
 
 // Organizations returns the discovery organizations
 // The second return value is a boolean that indicates whether a fresh list was updated internally
 // If there was an error, a cached copy is returned if available.
-func (discovery *Discovery) Organizations(ctx context.Context) (*Organizations, bool, error) {
-	if !discovery.DetermineOrganizationsUpdate() {
-		return &discovery.OrganizationList, false, nil
+// cache is set to true if there should be no network call done
+func (discovery *Discovery) Organizations(ctx context.Context, cache bool) (*Organizations, bool, error) {
+	discovery.mu.RLock()
+	if cache || !discovery.determineOrganizationsUpdate() {
+		discovery.mu.RUnlock()
+		return discovery.cachedOrgs(), false, nil
 	}
+	discovery.mu.RUnlock()
+	discovery.mu.Lock()
+	defer discovery.mu.Unlock()
 	file := "organization_list.json"
 	var jsonDecode Organizations
 	update, err := discovery.file(ctx, file, discovery.OrganizationList.Version, discovery.OrganizationList.UpdateHeader, &jsonDecode)
@@ -338,22 +337,18 @@ func (discovery *Discovery) Organizations(ctx context.Context) (*Organizations,
 		statErr := &httpw.StatusError{}
 		if errors.As(err, &statErr) {
 			if statErr.Status != 304 {
-				log.Logger.Warningf("failed to get fresh organizations: %v", err)
+				slog.Warn("failed to get fresh organization", "error", err)
 			} else {
 				discovery.OrganizationList.Timestamp = time.Now()
-				log.Logger.Debugf("got 304 for discovery, organization_list.json not modified")
+				slog.Debug("got HTTP 304 for discovery, organization_list.json is not modified")
 				err = nil
 			}
 		}
 		// Return previous with an error
-		orgs, perr := discovery.previousOrganizations()
-		if perr != nil {
-			log.Logger.Warningf("failed to get previous discovery organizations: %v", perr)
-		}
-		return orgs, false, err
+		return discovery.cachedOrgs(), false, err
 	}
 	if len(jsonDecode.List) == 0 {
-		log.Logger.Warningf("fresh organization list is empty")
+		slog.Warn("fresh organization list is empty")
 	} else {
 		discovery.OrganizationList = jsonDecode
 	}
@@ -364,13 +359,26 @@ func (discovery *Discovery) Organizations(ctx context.Context) (*Organizations,
 	return &discovery.OrganizationList, true, nil
 }
 
+func (discovery *Discovery) cachedServers() *Servers {
+	if discovery.ServerList.Version == 0 {
+		return nil
+	}
+	return &discovery.ServerList
+}
+
 // Servers returns the discovery servers
 // The second return value is a boolean that indicates whether a fresh list was updated internally
 // If there was an error, a cached copy is returned if available.
-func (discovery *Discovery) Servers(ctx context.Context) (*Servers, bool, error) {
-	if !discovery.DetermineServersUpdate() {
-		return &discovery.ServerList, false, nil
+// cache is set to true if there should be no network call done
+func (discovery *Discovery) Servers(ctx context.Context, cache bool) (*Servers, bool, error) {
+	discovery.mu.RLock()
+	if cache || !discovery.determineServersUpdate() {
+		discovery.mu.RUnlock()
+		return discovery.cachedServers(), false, nil
 	}
+	discovery.mu.RUnlock()
+	discovery.mu.Lock()
+	defer discovery.mu.Unlock()
 	file := "server_list.json"
 	var jsonDecode Servers
 	update, err := discovery.file(ctx, file, discovery.ServerList.Version, discovery.ServerList.UpdateHeader, &jsonDecode)
@@ -378,22 +386,18 @@ func (discovery *Discovery) Servers(ctx context.Context) (*Servers, bool, error)
 		statErr := &httpw.StatusError{}
 		if errors.As(err, &statErr) {
 			if statErr.Status != 304 {
-				log.Logger.Warningf("failed to get fresh servers: %v", err)
+				slog.Warn("failed to get fresh servers", "error", err)
 			} else {
 				discovery.ServerList.Timestamp = time.Now()
-				log.Logger.Debugf("got 304 for discovery, server_list.json not modified")
+				slog.Debug("got HTTP 304 for discovery, server_list.json is not modified")
 				err = nil
 			}
 		}
 		// Return previous with an error
-		srvs, perr := discovery.previousServers()
-		if perr != nil {
-			log.Logger.Warningf("failed to get previous discovery servers: %v", perr)
-		}
-		return srvs, false, err
+		return discovery.cachedServers(), false, err
 	}
 	if len(jsonDecode.List) == 0 {
-		log.Logger.Warningf("fresh server list is empty")
+		slog.Warn("fresh server list is empty")
 	} else {
 		discovery.ServerList = jsonDecode
 	}
@@ -404,23 +408,37 @@ func (discovery *Discovery) Servers(ctx context.Context) (*Servers, bool, error)
 	return &discovery.ServerList, true, nil
 }
 
-func (discovery *Discovery) UpdateServers(other Discovery) {
-	if other.ServerList.Version >= discovery.ServerList.Version {
-		discovery.ServerList = other.ServerList
+// UpdateServers updates the discovery servers to the new version
+// It does this by checking versions
+func (discovery *Discovery) UpdateServers(other Servers) {
+	if other.Version >= discovery.ServerList.Version {
+		discovery.ServerList = other
 	}
 }
 
-func (discovery *Discovery) Copy() (Discovery, error) {
-	var dest Discovery
-	b, err := json.Marshal(discovery)
-	if err != nil {
-		return dest, err
+// UpdateOrganizations updates the discovery organizations to the new version
+// It does this by checking versions
+func (discovery *Discovery) UpdateOrganizations(other Organizations) {
+	if other.Version >= discovery.OrganizationList.Version {
+		discovery.OrganizationList = other
 	}
+}
 
-	err = json.Unmarshal(b, &dest)
-	if err != nil {
-		return dest, err
+// Fill makes sure that the cache is filled with the embedded discovery
+func (discovery *Discovery) Fill() error {
+	if !HasCache {
+		return nil
 	}
 
-	return dest, nil
+	discovery.mu.Lock()
+	defer discovery.mu.Unlock()
+
+	var err error
+	var es Servers
+	err = errors.Join(err, json.Unmarshal(eServers, &es))
+	discovery.UpdateServers(es)
+	var eo Organizations
+	err = errors.Join(err, json.Unmarshal(eOrganizations, &eo))
+	discovery.UpdateOrganizations(eo)
+	return err
 }
