@@ -3,9 +3,8 @@ package failover
 import (
 	"context"
 	"fmt"
+	"log/slog"
 	"time"
-
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
 )
 
 type sender interface {
@@ -44,7 +43,7 @@ func (m *DroppedConMon) dropped(startBytes int64) (bool, error) {
 	if err != nil {
 		return false, err
 	}
-	log.Logger.Debugf("[Failover] Alive check, current Rx bytes: %d, start Rx bytes: %d", b, startBytes)
+	slog.Debug("Failover alive check", "rx bytes", b, "start bytes", startBytes)
 	return b <= startBytes, nil
 }
 
@@ -71,17 +70,17 @@ func (m *DroppedConMon) Start(ctx context.Context, gateway string, mtuSize int)
 	// Send a ping and wait for max 2 seconds
 	// If we have then increased Rx bytes we return early
 	if err = p.Send(1); err != nil {
-		log.Logger.Debugf("[Failover] First ping failed, exiting...")
+		slog.Debug("Failover first ping failed, exiting...")
 		return false, err
 	}
-	log.Logger.Debugf("[Failover] Now we are doing alive check")
+	slog.Debug("Failover doing alive check")
 
 	// Read the pong, if we got the echo reply then everything is fine, early return
 	if err = p.Read(time.Now().Add(m.pInterval)); err == nil {
-		log.Logger.Debugf("[Failover] Got early pong, exiting...")
+		slog.Debug("Failover got early pong, exiting...")
 		return false, err
 	}
-	log.Logger.Debugf("[Failover] Error reading pong: %v", err)
+	slog.Debug("Failover error reading pong", "error", err)
 
 	// Create a new ticker that executes our ping function every 'interval' seconds
 	// It starts immediately and stops when we reach the end
@@ -89,15 +88,15 @@ func (m *DroppedConMon) Start(ctx context.Context, gateway string, mtuSize int)
 	defer ticker.Stop()
 
 	// Otherwise send n pings, without waiting for pong and then check if dropped
-	log.Logger.Debugf("[Failover] Starting by sending pings and not waiting for pong...")
+	slog.Debug("Failover started by sending pings and not waiting for a pong...")
 	// Loop until the max drop counter
 	// We begin with 2 as this is used as the sequence number for ping
 	// and we have already sent a ping
 	for s := 2; s <= m.pDropped; s++ {
-		log.Logger.Debugf("[Failover] Sending ping: %d, with size: %d", s, mtuSize)
+		slog.Debug("Failover sending ping", "ping", s, "size", mtuSize)
 		// Send a ping and return if an error occurs
 		if err := p.Send(s); err != nil {
-			log.Logger.Debugf("[Failover] A ping failed, exiting...")
+			slog.Debug("Failover ping failed, exiting...")
 			return false, err
 		}
 		// Wait for the next tick to continue
