@@ -7,6 +7,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"log/slog"
 	"net/http"
 	"net/url"
 	"path"
@@ -203,6 +204,8 @@ func (c *Client) Do(ctx context.Context, method string, urlStr string, opts *Opt
 	ctx, cncl := context.WithTimeout(ctx, timeout)
 	defer cncl()
 
+	slog.Debug("sending request", "method", method, "url", urlStr)
+
 	// Create request object with the body reader generated from the optional arguments
 	req, err := http.NewRequestWithContext(ctx, method, urlStr, optionalBodyReader(opts))
 	if err != nil {
