@@ -1,3 +1,4 @@
+// Package levenshtein implements the algorithm for calculating the "distance" between two strings
 package levenshtein
 
 import (
@@ -27,9 +28,9 @@ func levenshtein(os, ot string) int {
 	}
 
 	// loop through every word in the first string
-	for i := 0; i < n; i++ {
+	for i := range n {
 		v1[0] = i + 1
-		for j := 0; j < m; j++ {
+		for j := range m {
 			// calculate deletion cost,
 			// insertion cost and
 			// substitution cost to get from the string
