@@ -2,11 +2,11 @@ package server
 
 import (
 	"context"
+	"log/slog"
 	"time"
 
 	"codeberg.org/eduVPN/eduvpn-common/internal/api"
 	"codeberg.org/eduVPN/eduvpn-common/internal/config/v2"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
 	"codeberg.org/eduVPN/eduvpn-common/types/server"
 	"github.com/jwijenbergh/eduoauth-go"
 )
@@ -45,7 +45,7 @@ func (s *Servers) AddCustom(ctx context.Context, id string, ot *int64) error {
 		// authorization has failed, remove the server again
 		rerr := s.config.RemoveServer(id, server.TypeCustom)
 		if rerr != nil {
-			log.Logger.Warningf("could not remove custom server: '%s' after failing authorization: %v", id, rerr)
+			slog.Warn("could not remove custom server after failing authorization", "server", id, "error", rerr)
 		}
 		return err
 	}
