@@ -3,33 +3,56 @@ package server
 import (
 	"context"
 	"errors"
+	"log/slog"
+	"net/url"
+	"strings"
 	"time"
 
 	"codeberg.org/eduVPN/eduvpn-common/internal/api"
 	"codeberg.org/eduVPN/eduvpn-common/internal/config/v2"
 	"codeberg.org/eduVPN/eduvpn-common/internal/discovery"
-	"codeberg.org/eduVPN/eduvpn-common/internal/log"
-	"codeberg.org/eduVPN/eduvpn-common/internal/util"
 	"codeberg.org/eduVPN/eduvpn-common/types/server"
 	"github.com/jwijenbergh/eduoauth-go"
 )
 
+// ReplaceWAYF replaces an authorization template containing of @RETURN_TO@ and @ORG_ID@ with the authorization URL and the organization ID
+// See https://github.com/eduvpn/documentation/blob/dc4d53c47dd7a69e95d6650eec408e16eaa814a2/SERVER_DISCOVERY_SKIP_WAYF.md
+func ReplaceWAYF(template string, authURL string, orgID string) string {
+	// We just return the authURL in the cases where the template is not given or is invalid
+	if template == "" {
+		return authURL
+	}
+	if !strings.Contains(template, "@RETURN_TO@") {
+		return authURL
+	}
+	if !strings.Contains(template, "@ORG_ID@") {
+		return authURL
+	}
+	// Replace authURL
+	template = strings.Replace(template, "@RETURN_TO@", url.QueryEscape(authURL), 1)
+
+	// If now there is no more ORG_ID, return as there weren't enough @ symbols
+	if !strings.Contains(template, "@ORG_ID@") {
+		return authURL
+	}
+	// Replace ORG ID
+	template = strings.Replace(template, "@ORG_ID@", url.QueryEscape(orgID), 1)
+	return template
+}
+
 // AddSecure adds a secure internet server
 // `ctx` is the context used for cancellation
 // `disco` are the discovery servers
 // `orgID` is the organiztaion ID
 // `ot` specifies specifies the start time OAuth was already triggered
-func (s *Servers) AddSecure(ctx context.Context, discom *discovery.Manager, orgID string, ot *int64) error {
+func (s *Servers) AddSecure(ctx context.Context, disco *discovery.Discovery, orgID string, ot *int64) error {
 	if s.config.HasSecureInternet() {
 		return errors.New("a secure internet server already exists")
 	}
-	disco, release := discom.Discovery(false)
 	dorg, dsrv, err := disco.SecureHomeArgs(orgID)
 	if err != nil {
-		release()
 		return err
 	}
-	release()
 
 	sd := api.ServerData{
 		ID:         dorg.OrgID,
@@ -37,17 +60,15 @@ func (s *Servers) AddSecure(ctx context.Context, discom *discovery.Manager, orgI
 		BaseWK:     dsrv.BaseURL,
 		BaseAuthWK: dsrv.BaseURL,
 		ProcessAuth: func(ctx context.Context, url string) (string, error) {
-			newd, release := discom.Discovery(true)
-			defer release()
 			// the only thing we can do is log warn
 			// this is already done in the functions
-			newd.Servers(ctx)       //nolint:errcheck
-			newd.Organizations(ctx) //nolint:errcheck
-			updorg, updsrv, err := newd.SecureHomeArgs(orgID)
+			disco.Servers(ctx, false)       //nolint:errcheck
+			disco.Organizations(ctx, false) //nolint:errcheck
+			updorg, updsrv, err := disco.SecureHomeArgs(orgID)
 			if err != nil {
 				return "", err
 			}
-			ret := util.ReplaceWAYF(updsrv.AuthenticationURLTemplate, url, updorg.OrgID)
+			ret := ReplaceWAYF(updsrv.AuthenticationURLTemplate, url, updorg.OrgID)
 			return ret, nil
 		},
 	}
@@ -76,7 +97,7 @@ func (s *Servers) AddSecure(ctx context.Context, discom *discovery.Manager, orgI
 		// authorization has failed, remove the server again
 		rerr := s.config.RemoveServer(orgID, server.TypeSecureInternet)
 		if rerr != nil {
-			log.Logger.Warningf("could not remove secure internet server: '%s' after failing authorization: %v", orgID, rerr)
+			slog.Warn("could not remove secure internet server after failing authorization", "server", orgID, "error", rerr)
 		}
 		return err
 	}
@@ -89,25 +110,21 @@ func (s *Servers) AddSecure(ctx context.Context, discom *discovery.Manager, orgI
 // `disco` are the discovery servers
 // `tok` are the tokens such that the server can be found without triggering auth
 // `disableAuth` is set to true when authorization should not be triggered
-func (s *Servers) GetSecure(ctx context.Context, orgID string, discom *discovery.Manager, tok *eduoauth.Token, disableAuth bool) (*Server, error) {
+func (s *Servers) GetSecure(ctx context.Context, orgID string, disco *discovery.Discovery, tok *eduoauth.Token, disableAuth bool) (*Server, error) {
 	srv, err := s.config.GetServer(orgID, server.TypeSecureInternet)
 	if err != nil {
 		return nil, err
 	}
 
-	disco, release := discom.Discovery(false)
 	dorg, dhome, err := disco.SecureHomeArgs(orgID)
 	if err != nil {
-		release()
 		return nil, err
 	}
 
 	dloc, err := disco.ServerByCountryCode(srv.CountryCode)
 	if err != nil {
-		release()
 		return nil, err
 	}
-	release()
 
 	sd := api.ServerData{
 		ID:         dorg.OrgID,
@@ -115,19 +132,17 @@ func (s *Servers) GetSecure(ctx context.Context, orgID string, discom *discovery
 		BaseWK:     dloc.BaseURL,
 		BaseAuthWK: dhome.BaseURL,
 		ProcessAuth: func(ctx context.Context, url string) (string, error) {
-			newd, release := discom.Discovery(true)
-			defer release()
 			// the only thing we can do is log warn
 			// this is already done in the functions
-			newd.MarkServersExpired()
-			newd.Servers(ctx) //nolint:errcheck
-			newd.MarkOrganizationsExpired()
-			newd.Organizations(ctx) //nolint:errcheck
-			updorg, updsrv, err := newd.SecureHomeArgs(orgID)
+			disco.MarkServersExpired()
+			disco.Servers(ctx, false) //nolint:errcheck
+			disco.MarkOrganizationsExpired()
+			disco.Organizations(ctx, false) //nolint:errcheck
+			updorg, updsrv, err := disco.SecureHomeArgs(orgID)
 			if err != nil {
 				return "", err
 			}
-			ret := util.ReplaceWAYF(updsrv.AuthenticationURLTemplate, url, updorg.OrgID)
+			ret := ReplaceWAYF(updsrv.AuthenticationURLTemplate, url, updorg.OrgID)
 			return ret, nil
 		},
 		DisableAuthorize: disableAuth,
