@@ -138,21 +138,12 @@ func (s *Server) connect(ctx context.Context, pTCP bool) (*srvtypes.Configuratio
 	if err != nil {
 		return nil, err
 	}
-	var proxy *srvtypes.Proxy
-	if apicfg.Proxy != nil {
-		proxy = &srvtypes.Proxy{
-			SourcePort: apicfg.Proxy.SourcePort,
-			ListenPort: apicfg.Proxy.ListenPort,
-			Peer:       apicfg.Proxy.Peer,
-		}
-	}
 	return &srvtypes.Configuration{
 		VPNConfig:        apicfg.Configuration,
 		Protocol:         apicfg.Protocol,
 		DefaultGateway:   chosenP.DefaultGateway,
 		DNSSearchDomains: chosenP.DNSSearchDomains,
 		ShouldFailover:   chosenP.ShouldFailover() && !pTCP,
-		Proxy:            proxy,
 	}, nil
 }
 
