@@ -24,12 +24,14 @@ func NewServer(handler http.Handler, listener net.Listener) *Server {
 	}
 
 	s := httptest.NewUnstartedServer(handler)
-	s.Listener.Close()
+	// This is only for tests and if this fails, bad luck
+	s.Listener.Close() //nolint:errcheck
 	s.Listener = listener
 	s.StartTLS()
 	return &Server{s}
 }
 
+// HandlerPath handles a specific HTTP query
 type HandlerPath struct {
 	Method          string
 	Path            string
@@ -38,6 +40,7 @@ type HandlerPath struct {
 	ResponseCode    int
 }
 
+// HandlerFunc handles a HTTP request
 func (hp *HandlerPath) HandlerFunc() func(http.ResponseWriter, *http.Request) {
 	if hp.ResponseHandler != nil {
 		return hp.ResponseHandler
