@@ -61,7 +61,7 @@ type RequiredAskTransition struct {
 	C *cookie.Cookie `json:"cookie,omitempty"`
 	// Data is the data associated to the transition, e.g. the list of profiles (Profiles struct)
 	// or the list of secure internet locations ([]string)
-	Data interface{} `json:"data"`
+	Data any `json:"data"`
 }
 
 // Expiry is the struct that gives the time at which certain expiry elements should be shown
@@ -164,18 +164,6 @@ type List struct {
 	Custom []Server `json:"custom_servers,omitempty"`
 }
 
-// Proxy defines the structure with the arguments that should be passed to start proxyguard
-type Proxy struct {
-	// SourcePort is the source port for the client TCP connection
-	SourcePort int `json:"source_port"`
-	// ListenPort is the port for the client UDP connection, this is the value that is set as the WireGuard endpoint prepended with "127.0.0.1:"
-	// So if the listen port is set to 1337, the WireGuard endpoint would be 127.0.0.1:1337
-	ListenPort int `json:"listen_port"`
-	// Peer is the URI of the upstream server
-	// Note that this exactly matches the "ProxyEndpoint" key in the WireGuard config
-	Peer string `json:"peer"`
-}
-
 // Configuration is the configuration that you get back when you call the get config function
 type Configuration struct {
 	// VPNConfig is the VPN Configuration, a WireGuard or OpenVPN Configuration
@@ -188,11 +176,9 @@ type Configuration struct {
 	DefaultGateway bool `json:"default_gateway"`
 	// DNSSearchDomains are the list of dns search domains
 	DNSSearchDomains []string `json:"dns_search_domains,omitempty"`
-	// ShouldFailover returns whether or not the client should attempt to failover
+	// ShouldFailover returns whether or not the client should start the online detection test to determine if failover is needed
+	// If the failover test returns "not dropped" then a "reconnect with TCP" button should be shown
 	ShouldFailover bool `json:"should_failover"`
-	// Proxy returns information for proxied VPN connections
-	// If this is non-nil a proxy must be started using StartProxyguard
-	Proxy *Proxy `json:"proxy,omitempty"`
 }
 
 // Current is the struct that defines the current server
