@@ -10,7 +10,7 @@ trap 'rm -rf -- "$tempdir"' EXIT
 pushd "$tempdir" >/dev/null
 
 echo "Getting translations..."
-wget -q -O translation.zip "https://hosted.weblate.org/download/eduvpn-common/eduvpn-common/?format=zip"
+wget -q -O translation.zip "https://hosted.weblate.org/download/eduvpn/eduvpn-common/?format=zip"
 unzip -q translation.zip
 
 echo "Copying translations..."
